# -*- coding: utf-8 -*-

import netCDF4
import pandas as pd
import numpy as np
import os
from datetime import datetime
import json
from functions import log, json_converter, addDataNetCDF
from envass import qualityassurance 


class skintemp(object):
    def __init__(self):
        self.id = ""
        self.folder = ""
        self.data = {}
        self.general_attributes = {
            "institution": "EPFL",
            "source": "Skin Temperature Measurements",
            "references": "LéXPLORE common instruments camille.minaudo@epfl.ch>",
            "history": "See history on Renku",
            "conventions": "CF 1.7",
            "comment": "Skin temperature measurements collected on Lexplore Platform in Lake Geneva",
            "title": "Lexplore Skin Temperature Measurements"
        }
        self.dimensions = {
            'time': {'dim_name': 'time', 'dim_size': None}
        }
        self.variables = {
            'time': {'var_name': 'time', 'dim': ('time',), 'unit': 'seconds since 1970-01-01 00:00:00','longname': 'time'},
            'skintemp': {'var_name': 'skintemp', 'dim': ('time',), 'unit': 'degC', 'longname': 'skin temperature',},
            'skytemp': {'var_name': 'skytemp', 'dim': ('time',), 'unit': 'degC', 'longname': 'sky temperature'},
               }

    def read_data(self, folder):
        ''' Read and format raw data:
            Input: 
                folder: (string): Path to the raw data folder (L0)
            '''

        if os.path.isfile(os.path.join(folder, "LeXPLORE_WEST_KT1585.dat")):
            file = os.path.join(folder, "LeXPLORE_WEST_KT1585.dat")
        else:
            log("Cannot find skin temperature file", 3)
            return False
        log("Reading skin temperature data from: "+file, 3)
        try:
            df = pd.read_csv(file, header=None, encoding = "ISO-8859-1", skiprows=4)
            df.columns = ["date","record","skintemp","skytemp"]
            df["time"] = df["date"].apply(lambda x: datetime.timestamp(datetime.strptime(x, '%Y-%m-%d %H:%M:%S')))
            #Clean duplicated data 
            df.drop_duplicates(subset = 'date', inplace = True)
            df.sort_values("time", inplace=True)
            df.reset_index(inplace=True, drop=True)

            df["datetime"] = pd.to_datetime(df['time'], unit='s')
            df["skintemp"] = pd.to_numeric(df["skintemp"], errors='coerce')
            df["skytemp"] = pd.to_numeric(df["skytemp"], errors='coerce')
            self.data = df
            self.time = df["time"]
            log("Successfully read data", 3)
        except Exception as e:
            log("Failed to parse data", e)
            return False
        return True

    def to_NetCDF(self, folder, prefix):
        '''
            Create or update a netCDF file based on the imported data
            
            input:
                folder (string): Path to the output folder
                prefix (string): prefix of the output filename
            '''

        log("Writing "+prefix+" data to NetCDF", 3)
        dt_min = np.nanmin(self.data["time"])
        dt_max = np.nanmax(self.data["time"])
        diff = round((dt_max - dt_min)/(24*3600*7))
        for i in range(diff + 1):
            d = datetime.utcfromtimestamp(dt_min + i * (24*3600*7))
            d1 = datetime.utcfromtimestamp(dt_min + (i + 1) * (24*3600*7))
            dt = d.strftime('%Y%m%d')

            data = self.data[(self.data['datetime'] >= d) & (self.data['datetime'] < d1)]

            if len(data) == 0:
                continue
                
            log("Writing " + dt + " to NetCDF", 4)
            if not os.path.exists(folder):
                os.makedirs(folder)
            filename = "_".join([prefix, "Skin_Temperature", dt+".nc"])
            self.outfile = os.path.join(folder, filename)
            if os.path.isfile(self.outfile):
                nc = netCDF4.Dataset(self.outfile,mode='a',format='NETCDF4')
                if data["time"].iloc[-1] in nc.variables['time'][:]:
                    log("Duplicated run, no data added", 4)
                    nc.close() 
                    continue
                
                else:
                    time_file = np.array(nc.variables['time'][:])
                    idx = np.where(self.time==time_file[-1])[0][0] + 1

                    time = nc.variables['time']
                    time[:] = np.append(time[:], self.time[idx:])
                    
                    for key, values in self.variables.items():
                        addDataNetCDF(np.array(data[key]),idx,nc,**values)
                    nc.close()

            else: 

                nc = netCDF4.Dataset(self.outfile, mode='w', format='NETCDF4')

                for key in self.general_attributes:
                    setattr(nc, key, self.general_attributes[key])

                for key, values in self.dimensions.items():
                    nc.createDimension(values['dim_name'], values['dim_size'])

                for key, values in self.variables.items():
                    var = nc.createVariable(values["var_name"], np.float64, values["dim"], fill_value=np.nan)
                    var.units = values["unit"]
                    var.long_name = values["longname"]
                    var[:] = np.array(data[key])

                nc.close()
            log("Successfully wrote NetCDF file", 3)

    def quality_flags(self, file_path, simple=True):
        '''
            Quality assurance for timeseries data
            
            input:
                file_path (string): path to the quality_assurance.json file, containing the quality check to perform
                simple (bool): - True: only 'simple' checks are performed (is numeric, bounds, edges)
                               - False: all checks are performed on the selected timeserie (not recommended)
            '''
        quality_assurance_dict = json_converter(json.load(open(file_path)))
        for key, values in self.variables.copy().items():
            if (quality_assurance_dict[key]["advanced"]) or (quality_assurance_dict[key]["simple"]):
                name = key + "_qual"
                self.variables[name] = {'var_name': name, 'dim': values["dim"],
                            'unit': '0 = nothing to report, 1 = more investigation',
                            'longname': name, }
                if simple == True: 
                    self.data[name]= qualityassurance(np.array(self.data[key]), np.array(self.data["time"]), **quality_assurance_dict[key]["simple"])
                else: 
                    quality_assurance_all = dict(quality_assurance_dict[key]["simple"], **quality_assurance_dict[key]["advanced"])
                    self.data[name]= qualityassurance(np.array(self.data[key]), np.array(self.data["time"]), **quality_assurance_all)


